{"record": {"public_notes": [{"source": "arXiv", "value": "18 pages, 13 figures"}], "page_nr": "18", "copyright": [{"material": "", "holder": "", "statement": "", "year": ""}], "license": [{"url": "http://arxiv.org/licenses/nonexclusive-distrib/1.0/", "material": "", "license": ""}], "subject_terms": [], "arxiv_eprints": [{"categories": ["cond-mat.quant-gas", "hep-lat", "hep-th", "nucl-th", "quant-ph"], "value": "1605.07085"}], "titles": [{"source": ["arXiv"], "subtitle": "", "title": "The entanglement spectrum and R\\'enyi entropies of non-relativistic\n  conformal fermions"}], "authors": [{"affiliations": "", "surname": "Porter", "given_names": "William J.", "full_name": "Porter, William J."}, {"affiliations": "", "surname": "Drut", "given_names": "Joaqu\u00edn E.", "full_name": "Drut, Joaqu\u00edn E."}], "external_system_numbers": [{"institute": "arXiv", "value": "oai:arXiv.org:1605.07085"}], "publication_info": [{"note": "arXiv", "page_artid": "", "journal_title": "", "pubinfo_freetext": "", "journal_volume": "", "year": "", "journal_issue": ""}], "abstracts": [{"source": ["arXiv"], "value": "We characterize non-perturbatively the R\\'enyi entropies of degree n=2,3,4,and 5 of three-dimensional, strongly coupled many-fermion systems in thescale-invariant regime of short interaction range and large scattering length,i.e. in the unitary limit. We carry out our calculations using lattice methodsdevised recently by us. Our results show the effect of strong pairingcorrelations on the entanglement entropy, which modify the sub-leading behaviorfor large subsystem sizes (as characterized by the dimensionless parameter x=kFL_A, where kF is the Fermi momentum and L_A the linear subsystem size), butleave the leading order unchanged relative to the non-interacting case.Moreover, we find that the onset of the sub-leading asymptotic regime is atsurprisingly small x=2-4. We provide further insight into the entanglementproperties of this system by analyzing the spectrum of the entanglementHamiltonian of the two-body problem from weak to strong coupling. The low-lyingentanglement spectrum displays clear features as the strength of the couplingis varied, such as eigenvalue crossing, a sharp change in the Schmidt gap, andscale invariance at unitarity. Beyond the low-lying component, the spectrumappears as a quasi-continuum distribution, for which we present a statisticalcharacterization; we find, in particular, that the mean shifts to infinity asthe coupling is turned off, which indicates that that part of the spectrumrepresents non-perturbative contributions to the entanglement Hamiltonian. Incontrast, the low-lying entanglement spectrum evolves to finite values in thenoninteracting limit. The scale invariance of the unitary regime guaranteesthat our results are universal features intrinsic to 3D quantum mechanics andrepresent a well-defined prediction for ultracold atom experiments, which wererecently shown to have direct access to the entanglement entropy."}], "imprints": [{"date": "2016-05-23"}]}, "errors": [], "file_name": "desy_records.xml", "source_data": "<xml><record></record></xml>"}
{"record": {"files": [], "preprint_date": "2017-01-09", "page_nr": ["54"], "acquisition_source": {"date": "2017-01-11T15:31:34.283678", "source": "arXiv", "method": "arXiv", "submission_number": "a5b8c49ed80a11e6973502163e0191a4"}, "license": [{"url": "http://arxiv.org/licenses/nonexclusive-distrib/1.0/", "license": "arXiv-1.0"}], "public_notes": [{"source": "arXiv", "value": "54 pages, 2 figures"}], "authors": [{"affiliations": [], "surname": "Ranocha", "given_names": "Hendrik", "full_name": "Ranocha, Hendrik"}], "field_categories": [{"source": "publisher", "term": "math.NA", "scheme": "ARXIV"}], "titles": [{"source": "arXiv", "subtitle": "", "title": "Comparison of some Entropy Conservative Numerical Fluxes for the Euler Equations"}], "collections": [{"primary": "HEP"}, {"primary": "Citeable"}, {"primary": "arXiv"}], "arxiv_eprints": [{"categories": ["math.NA"], "value": "1701.02264"}], "external_system_numbers": [{"institute": "arXiv", "value": "oai:arXiv.org:1701.02264"}], "copyright": [{"material": "", "holder": "", "statement": "", "year": ""}], "abstracts": [{"source": "arXiv", "value": "Entropy stability is a well-known design principle for numerical methods in gas dynamics. Entropy conservative numerical fluxes can be used as ingredients in two kinds of schemes: Firstly, as building blocks in the subcell flux differencing form of Fisher and Carpenter (2013) and secondly (enhanced by dissipation) as numerical surface fluxes in finite volume like schemes. In this article, the flux differencing theory is extended, guaranteeing high-order for symmetric and consistent numerical fluxes. Additionally, an extension to simplices using a genuinely multi-dimensional framework of summation-by-parts operators is investigated for the first time in this framework. Moreover, several new entropy conservative numerical fluxes are developed and compared extensively with existing ones, both in theory and in numerical tests, using the flux differencing and the finite volume framework."}], "imprints": [{"date": ""}]}, "errors": [], "file_name": "desy_records.xml", "source_data": "<xml><record></record></xml>"}
